package fxKirjalista;

import java.io.PrintStream;
import java.net.URL;
import java.util.Optional;
import java.util.ResourceBundle;

import Kirjalista.Kirja;
import Kirjalista.Kirjasto;
import fi.jyu.mit.fxgui.Dialogs;
import fi.jyu.mit.fxgui.ListChooser;
import fi.jyu.mit.fxgui.TextAreaOutputStream;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.layout.Pane;
import javafx.scene.control.ButtonType;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextArea;
import javafx.scene.text.Font;
import javafx.stage.Modality;
import javafx.stage.Stage;


/**
 * Ohjelman pääikkunan käsittelijä
 */
public class KirjalistaGUIController implements Initializable {
    
    @FXML private ListChooser<Kirja> chooserKirjat;
    @FXML private ScrollPane paneKirja;
    
    /**
     * Kirjan lisäys
     */
    @FXML void handleLisaaKirja() {
        try {
            Stage lisaaStage = new Stage();
            lisaaStage.initModality(Modality.APPLICATION_MODAL);
            
            final FXMLLoader ldr = new FXMLLoader(getClass().getResource("LisaaView.fxml"));
            final Pane root = (Pane)ldr.load();
            
            final LisaaController lisaaCtrl = (LisaaController) ldr.getController();
            final Scene scene = new Scene(root);
            scene.getStylesheets().add(getClass().getResource("kirjalista.css").toExternalForm());
            lisaaStage.setScene(scene);
            lisaaStage.setTitle("Lisää kirja");
            
            lisaaCtrl.setIsanta(this);
            lisaaCtrl.setKirjasto(this.kirjasto);

            lisaaStage.show();

        } catch(Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Kirjan muokkaus
     */
    @FXML void handleMuokkaaKirja() {
        try {
            Stage muokkaaStage = new Stage();
            muokkaaStage.initModality(Modality.APPLICATION_MODAL);
            
            final FXMLLoader ldr = new FXMLLoader(getClass().getResource("MuokkaaView.fxml"));
            final Pane root = (Pane)ldr.load();
            
            final LisaaController muokkaaCtrl = (LisaaController) ldr.getController();
            final Scene scene = new Scene(root);
            scene.getStylesheets().add(getClass().getResource("kirjalista.css").toExternalForm());
            muokkaaStage.setScene(scene);
            muokkaaStage.setTitle("Muokkaa kirjaa");
            
            muokkaaCtrl.setIsanta(this);
            muokkaaCtrl.setKirjasto(this.kirjasto);

            muokkaaStage.show();

        } catch(Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * Tiedosto-valikon tallenna-valinta
     */
    @FXML void handleTallenna() {
        // TODO:
        Dialogs.showMessageDialog("Vielä ei osata tallentaa");

    }

    /**
     * Apua-valikon tietoja-valinta
     */
    @FXML void handleTietoja() {
        // TODO:
        Dialogs.showMessageDialog("Vielä ei osata antaa tietoja");

    }

    /**
     * Tiedosto-valikon tulosta-valinta
     */
    @FXML void handleTulosta() {
        // TODO:
        Dialogs.showMessageDialog("Vielä ei osata tulostaa");
    }
    
    /**
     * Muokkaa-valikon poista-valinta
     */
    @FXML void poistaKirja() {
         poistoVahvistus();
    }
    
    @FXML void handleHaku() {
        Dialogs.showMessageDialog("Vielä ei osata hakea");
    }

    
// ============================================================================    
    
    private Kirjasto kirjasto;
    private Kirja valittuKirja;
    private TextArea areaKirja = new TextArea();
    
    /**
     * Luo näytölle dialogin, joka kysyy vahvistusta kirjan poistamiseksi
     */
    public static void poistoVahvistus() {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Poista");
        alert.setHeaderText(null);
        alert.setContentText("Haluatko varmasti poistaa kirjan?");

        ButtonType buttonTypeYes = new ButtonType("Kyllä", ButtonData.OK_DONE);
        ButtonType buttonTypeCancel = new ButtonType("Ei", ButtonData.CANCEL_CLOSE);

        alert.getButtonTypes().setAll(buttonTypeYes, buttonTypeCancel);

        Optional<ButtonType> result = alert.showAndWait();
        if ( result.get() == buttonTypeYes ) Dialogs.showMessageDialog("Vielä ei osata poistaa");
    }

    
    /**
     * Asettaa controllerille viitteen kirjasto-olioon
     * @param kirjasto Kirjasto-luokan olio, johon viite asetetaan
     */
    public void setKirjasto(Kirjasto kirjasto) {
        this.kirjasto = kirjasto;
    }
    
    
    /**
     * Hakee lisätyn kirjan sen id:n perusteella ja lisää sen näyttöön
     * @param idnro Kirjan id
     */
    public void hae(int idnro) {
        chooserKirjat.clear();
        
        int index = 0;
        for (int i = 0; i < kirjasto.getKirjaLkm(); i++) {
            Kirja kirja = kirjasto.haeKirja(i);
            if (kirja.getId() == idnro) index = i;
            chooserKirjat.add(kirja.getNimi(), kirja);
        }
        chooserKirjat.setSelectedIndex(index);
    }
    
    
    /**
     * Näyttää listasta valitun kirjan tiedot, 
     * tilapäisesti yhteen isoon edit-kenttään
     */
    protected void naytaJasen() {
        valittuKirja = chooserKirjat.getSelectedObject();

        if (valittuKirja == null) return;

        areaKirja.setText("");
        try (PrintStream os = TextAreaOutputStream.getTextPrintStream(areaKirja)) {
            valittuKirja.tulosta(os);
        }
    }
    
    /**
     * Tekee tarvittavat muut alustukset, nyt vaihdetaan GridPanen tilalle
     * yksi iso tekstikenttä, johon voidaan tulostaa jäsenten tiedot.
     * Alustetaan myös jäsenlistan kuuntelija 
     */
    protected void alusta() {
        paneKirja.setContent(areaKirja);
        areaKirja.setFont(new Font("Courier New", 12));
        paneKirja.setFitToHeight(true);
        
        chooserKirjat.clear();
        chooserKirjat.addSelectionListener(e -> naytaJasen());
    }

    @Override
    public void initialize(URL arg0, ResourceBundle arg1) {
        alusta();
        
    }


}
